@@ -33,8 +33,12 @@
 #define DEFAULT_MIN_FRAMES 0
 #define RTP_SBC_HEADER_TOTAL (12 + RTP_SBC_PAYLOAD_HEADER_SIZE)
 
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+/* BEGIN: Packing for rtp_payload */
+#ifdef _MSC_VER
+#pragma pack(push, 1)
+#endif
 
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
 /* FIXME: this seems all a bit over the top for a single byte.. */
 struct rtp_payload
 {
@@ -43,10 +47,8 @@ struct rtp_payload
   guint8 is_last_fragment:1;
   guint8 is_first_fragment:1;
   guint8 is_fragmented:1;
-} __attribute__ ((packed));
-
+}
 #elif G_BYTE_ORDER == G_BIG_ENDIAN
-
 struct rtp_payload
 {
   guint8 is_fragmented:1;
@@ -54,12 +56,19 @@ struct rtp_payload
   guint8 is_last_fragment:1;
   guint8 rfa0:1;
   guint8 frame_count:4;
-} __attribute__ ((packed));
-
+}
 #else
 #error "Unknown byte order"
 #endif
 
+#ifdef _MSC_VER
+;
+#pragma pack(pop)
+#else
+__attribute__ ((packed));
+#endif
+/* END: Packing for rtp_payload */
+
 enum
 {
   PROP_0,
@@ -284,10 +293,10 @@ gst_rtp_sbc_pay_class_init (GstRtpSBCPayClass * klass)
           "(-1 for maximum allowed by the mtu)",
           -1, G_MAXINT, DEFAULT_MIN_FRAMES, G_PARAM_READWRITE));
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_sbc_pay_sink_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_sbc_pay_src_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_sbc_pay_sink_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_sbc_pay_src_factory);
 
   gst_element_class_set_static_metadata (element_class, "RTP packet payloader",
       "Codec/Payloader/Network", "Payload SBC audio as RTP packets",
