@@ -370,7 +370,7 @@ GST_START_TEST (test_play_twice)
   /* cleanup */
   g_main_loop_unref (main_loop);
   gst_consistency_checker_free (consist);
-  gst_event_ref (play_seek_event);
+  gst_event_unref (play_seek_event);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
   gst_object_unref (bin);
@@ -473,7 +473,7 @@ GST_START_TEST (test_play_twice_then_add_and_play_again)
 
   /* cleanup */
   g_main_loop_unref (main_loop);
-  gst_event_ref (play_seek_event);
+  gst_event_unref (play_seek_event);
   gst_consistency_checker_free (consist);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
@@ -930,8 +930,8 @@ GST_START_TEST (test_loop)
 
   seek_event = gst_event_new_seek (1.0, GST_FORMAT_TIME,
       GST_SEEK_FLAG_SEGMENT | GST_SEEK_FLAG_FLUSH,
-      GST_SEEK_TYPE_SET, (GstClockTime) 0,
-      GST_SEEK_TYPE_SET, (GstClockTime) 1 * GST_SECOND);
+      GST_SEEK_TYPE_SET, (GstClockTime) 0, GST_SEEK_TYPE_SET,
+      (GstClockTime) 2 * GST_SECOND);
 
   main_loop = g_main_loop_new (NULL, FALSE);
   g_signal_connect (bus, "message::segment-done",
@@ -947,7 +947,9 @@ GST_START_TEST (test_loop)
   ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 
   /* wait for completion */
-  state_res = gst_element_get_state (bin, NULL, NULL, GST_CLOCK_TIME_NONE);
+  state_res =
+      gst_element_get_state (GST_ELEMENT (bin), NULL, NULL,
+      GST_CLOCK_TIME_NONE);
   ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
 
   res = gst_element_send_event (bin, seek_event);
@@ -1017,6 +1019,8 @@ GST_START_TEST (test_flush_start_flush_stop)
 
   /* cleanup */
   gst_element_set_state (pipeline, GST_STATE_NULL);
+  gst_object_unref (sinkpad1);
+  gst_object_unref (sinkpad2);
   gst_object_unref (pipeline);
 }
 
